#! /usr/bin/env python
from __future__ import print_function, annotations

import dataclasses
import os
from functools import cached_property
from pathlib import Path
import shutil
import urllib.parse as urlparse
import urllib.request as urlrequest
from typing import Optional
from xml.dom import minidom
import src.gs_netsuite_api.ns_utils as ns_utils

INDENT_LEVEL = 2


@dataclasses.dataclass
class ArgsDT:
    wsdl_url: str = ""
    output_dir: str = ""


def xml_to_pretty_string(xml):
    text = xml.toprettyxml(indent=" " * INDENT_LEVEL)
    not_empty_lines = [line for line in (text.splitlines()) if (line and not line.isspace())]
    return "\n".join(not_empty_lines)


@dataclasses.dataclass
class XsdDocument:
    uri: str
    output: Path
    parent: Optional[XsdDocument] = None

    @cached_property
    def wsdl_doc(self) -> minidom.Document:
        return self.read_xml_from_url()

    def read_xml_from_url(self) -> minidom.Document:
        print("Reading", urlparse.urlunparse(self.remote_url))
        text = urlrequest.urlopen(urlparse.urlunparse(self.remote_url)).read()
        return minidom.parseString(text)

    @cached_property
    def remote_url(self) -> urlparse.ParseResult:
        xsd_url_parsed = urlparse.urlparse(self.uri)
        if not xsd_url_parsed.scheme or not xsd_url_parsed.netloc:
            # It'a a relative path to wsdl_info.wsdl_url
            path_wsdl, filename = os.path.split(self.parent.remote_url.path)
            return urlparse.ParseResult(
                scheme=self.parent.remote_url.scheme,
                netloc=self.parent.remote_url.netloc,
                params=xsd_url_parsed.params,
                query=xsd_url_parsed.query,
                fragment=xsd_url_parsed.fragment,
                path=os.path.join(path_wsdl, self.uri),
            )
        return urlparse.urlparse(self.uri)

    def download(self):
        dir, filename = self.download_path()
        print("mkdir  ", dir)
        os.makedirs(dir, exist_ok=True)
        content = xml_to_pretty_string(self.read_xml_from_url())
        final_path = os.path.join(dir, filename)
        print("Writing", final_path)
        with open(final_path, "w") as f:
            f.write(content)

    def exists(self):
        return os.path.exists(os.path.join(*self.download_path()))

    def download_path(self) -> (str, str):
        wsdl_path, filename = os.path.split(urlparse.urlparse(self.uri).path)
        dir_dest = self.output.resolve()
        wsdl_path = wsdl_path.removeprefix(os.sep)
        wsdl_path = dir_dest.joinpath(wsdl_path)
        wsdl_path = wsdl_path.resolve()
        return wsdl_path, filename

    def create_child(self, uri: str) -> XsdDocument:
        return XsdDocument(uri=uri, parent=self, output=self.download_path()[0])


def download_xsd_imports(rootDoc: XsdDocument, xml_schema_mapping="xsd:"):
    xsd_import_elems = rootDoc.wsdl_doc.getElementsByTagName(xml_schema_mapping + "import")
    for elem in xsd_import_elems:
        schema_location = elem.attributes.get("schemaLocation")
        doc = rootDoc.create_child(uri=schema_location.value)
        if doc.exists():
            continue
        doc.download()
        download_xsd_imports(doc, xml_schema_mapping="")
        download_xsd_imports(doc, xml_schema_mapping="xsd:")


def main():
    ns = ns_utils.NetSuiteCredential.from_env_file("./auth_netsuite.env").get_netSuite()
    output_dir = Path("output").resolve()
    if output_dir.exists():
        shutil.rmtree(output_dir)
    rootDoc = XsdDocument(uri=ns.soap_api.wsdl_url, output=output_dir, parent=None)
    rootDoc.download()
    download_xsd_imports(rootDoc)
    download_xsd_imports(rootDoc, xml_schema_mapping="xsd:")


if __name__ == "__main__":
    main()



import dataclasses
import datetime
from pprint import pprint

import logging
from gs_netsuite_api.ns_utils import NetSuiteCredential, SearchParams
from gs_netsuite_api.api import ItemFulFillmentAPI


logging.root.setLevel(logging.INFO)
logging.root.addHandler(logging.StreamHandler())

# Question sur les datas

if __name__ == "__main__":
    # Existe d'autre methode pour avoir un NetSuiteCredential
    cred = NetSuiteCredential.from_env_file("./auth_netsuite.env")
    api = ItemFulFillmentAPI(credential=cred, search_params=SearchParams(page_size=5, nb_page=2))

    # api.get_one(6133226)

    all_ids = api.get_ids()
    print(len(all_ids), all_ids)

    data_all = api.get_all()
    pprint([dataclasses.asdict(p) for p in data_all])

    assert all_ids == [d.internalId for d in data_all]

    first_id = all_ids[0]
    data_one = api.get_one(first_id)
    pprint(data_one)

    multi_ids = all_ids[:3]
    data_multi = api.get_multi(multi_ids)
    pprint([dataclasses.asdict(p) for p in data_multi])

    ids_since = api.get_ids_since(datetime.datetime(2023, 1, 16))
    print(len(ids_since), ids_since)

    data_since = api.get_since(datetime.datetime(2023, 1, 16))
    print(len(data_since))
    pprint([dataclasses.asdict(p) for p in data_since])

    assert ids_since == [d.internalId for d in data_since]



from pprint import pprint

import logging
from gs_netsuite_api.ns_utils import NetSuiteCredential, SearchParams
from gs_netsuite_api.api import PurchaseAPI


logging.root.setLevel(logging.INFO)
logging.root.addHandler(logging.StreamHandler())

# Question sur les datas
# Le champs tax2Total c'est quoi ?
# custbody_stc_tax_after_discount ?
# custbody_stc_total_after_discount ?
# custbody_stc_amount_after_discount ?
# styatus vs orderStatus, orderStatus est vide

# Dans les lignes
# entre rate et amount ?
# taxAmount pas remplis mais tax1Amt oui ?
# custcolnv_po_purchase_type ? PO id 6133226 c'est Sales ? Achat pour ventes client en livraison direct chez client?
# quantityshiprec utilisation de quantityOnShipments car je trouve pas ce champ

if __name__ == "__main__":
    # Existe d'autre methode pour avoir un NetSuiteCredential
    cred = NetSuiteCredential.from_env_file("./auth_netsuite.env")
    api = PurchaseAPI(credential=cred, search_params=SearchParams(page_size=5, nb_page=2))

    pprint(api.get_one(2))

    # all_ids = api.get_ids()
    # print(len(all_ids), all_ids)
    #
    # data_all = api.get_all()
    # pprint([dataclasses.asdict(p) for p in data_all])
    #
    # assert all_ids == [d.internalId for d in data_all]
    #
    # first_id = all_ids[0]
    # data_one = api.get_one(first_id)
    # pprint(data_one)
    #
    # multi_ids = all_ids[:3]
    # data_multi = api.get_multi(multi_ids)
    # pprint([dataclasses.asdict(p) for p in data_multi])
    #
    # ids_since = api.get_ids_since(datetime.datetime(2023, 1, 16))
    # print(len(ids_since), ids_since)
    #
    # data_since = api.get_since(datetime.datetime(2023, 1, 16))
    # print(len(data_since))
    # pprint([dataclasses.asdict(p) for p in data_since])
    #
    # assert ids_since == [d.internalId for d in data_since]
#
